package info.textgrid.middleware;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.math.BigInteger;
import java.text.ParseException;
import java.util.Hashtable;
import java.util.Map;
import java.util.UUID;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.classicmayan.tools.ConedaKorID;
import org.classicmayan.tools.ImageMetsMods;
import org.classicmayan.tools.MediaHarvester;
import org.json.JSONArray;
import org.json.JSONException;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import info.textgrid.middleware.oaipmh.GetRecordType;
import info.textgrid.middleware.oaipmh.HeaderType;
import info.textgrid.middleware.oaipmh.ListIdentifiersType;
import info.textgrid.middleware.oaipmh.ListRecordsType;
import info.textgrid.middleware.oaipmh.MetadataType;
import info.textgrid.middleware.oaipmh.RecordType;
import info.textgrid.middleware.oaipmh.ResumptionTokenType;

public class IDIOMImages implements RecordDelivererInterface, RecordListDelivererInterface, IdentifierListDelivererInterface{
    private String kindID;
    private String perPage;
    private String pageNumber;
    private int totalHits;

    protected static Map<String, Integer> cursorCollector = new Hashtable<String, Integer>();
    protected static Map<String, String> fromUntilCollector = new Hashtable<String, String>();

    public IDIOMImages(){
    }

    /**
     * 
     * @throws JSONException
     * @throws IOException
     * @throws ParseException
     * @throws SAXException
     * @throws TransformerException
     * @throws TransformerFactoryConfigurationError
     */
    
    private String from;
    private String until;
    
    public ListRecordsType getRecords(String from, String to, String set, String resumptionToken) throws JSONException, IOException, ParseException, TransformerFactoryConfigurationError{
    	if(resumptionToken!=null) {
    		String fromUntil = fromUntilCollector.get(resumptionToken);
    		from = fromUntil.split(":")[0];
    		to = fromUntil.split(":")[1];
    	}
    
    	MediaHarvester mediaHarvester = new MediaHarvester(getKindID(),getPerPage(), getPageNumber(), from, to);
        JSONArray mediaList = mediaHarvester.getMediaListFromConedaKor().getJSONArray("records");
        ListRecordsType imageMetsModsList = new ListRecordsType();

        setTotalHits(mediaHarvester.getTotalMedia());  
        this.setFrom(from);
        this.setUntil(to);
        for(int i=0; i<mediaList.length(); i++){       
                ImageMetsMods imm = new ImageMetsMods(mediaList.getJSONObject(i));
                RecordType record = new RecordType();
                MetadataType metadataMets = new MetadataType();

                DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                DocumentBuilder builder;
                try {
                    builder = factory.newDocumentBuilder();
                    
                    Document doc = builder.parse(new InputSource(new StringReader(imm.getXML())));
                    metadataMets.setAny(doc.getDocumentElement());                   
                    record.setMetadata(metadataMets);
                    String dateStamp = mediaList.getJSONObject(i).get("created_at").toString();
                    String identifier = mediaList.getJSONObject(i).get("id").toString();
                    
                    record.setHeader(buildOAIPMH_RecordHeader(dateStamp, identifier));
                    imageMetsModsList.getRecord().add(record); 
                } catch (ParserConfigurationException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (SAXException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }    
        }
        imageMetsModsList.setResumptionToken(buildResumptionToken(resumptionToken, getTotalHits(), from, to));
        return imageMetsModsList;
    }

    public ListIdentifiersType processIdentifierList(String from, String to, String set, String resumptionToken) throws JSONException, IOException, ParseException{
    	if(resumptionToken!=null) {
    		String fromUntil = fromUntilCollector.get(resumptionToken);
    		from = fromUntil.split(":")[0];
    		to = fromUntil.split(":")[1];
    	}
        MediaHarvester mediaHarvester = new MediaHarvester(getKindID(),getPerPage(), getPageNumber(), from, to);
        JSONArray mediaList = mediaHarvester.getMediaListFromConedaKor().getJSONArray("records");
        ListIdentifiersType identifiersList = new ListIdentifiersType();

        setTotalHits(mediaHarvester.getTotalMedia());  
        
        for(int i=0; i<mediaList.length(); i++){       
            HeaderType header = new HeaderType();
            header.setDatestamp(mediaList.getJSONObject(i).get("created_at").toString());
            header.setIdentifier(mediaList.getJSONObject(i).get("id").toString());
            identifiersList.getHeader().add(header);
        }

        identifiersList.setResumptionToken(buildResumptionToken(resumptionToken, getTotalHits(), from, to));
        return identifiersList;
    }

    public GetRecordType getRecordById(String id) throws JSONException, IOException, ParseException{
        GetRecordType singleImageMetsMods = new GetRecordType();
        RecordType conedaKorRecord = new RecordType();
        MetadataType metadataMets = new MetadataType();

        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder;
        
        ImageMetsMods imm = new ImageMetsMods(new ConedaKorID(id));

        try {
            builder = factory.newDocumentBuilder();
            Document doc = builder.parse(new InputSource(new StringReader(imm.getXML())));
            metadataMets.setAny(doc.getDocumentElement());
            conedaKorRecord.setMetadata(metadataMets);
            conedaKorRecord.setHeader(buildOAIPMH_RecordHeader(imm.getCreationDate(), imm.getID()));
            singleImageMetsMods.setRecord(conedaKorRecord);
        } catch (ParserConfigurationException e) {
            // TODO Auto-generated catch block
            return null;
        } catch (SAXException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        return singleImageMetsMods;
    }

    /*public ListRecordsType getAllMedia() throws JSONException, IOException, ParseException, SAXException, TransformerFactoryConfigurationError, TransformerException{
        ListRecordsType imageMetsModsList = new ListRecordsType();
        MediaHarvester mediaHarvester = new MediaHarvester(getKindID(),getPerPage(), getPageNumber());
        int perPage = 100;
        setTotalHits(mediaHarvester.getTotalMedia());
        for(int j=1; j*perPage<=100 ;j++){
            mediaHarvester = new MediaHarvester(getKindID(),getPerPage(), Integer.toString(j));
            JSONArray mediaList = mediaHarvester.getMediaListFromConedaKor().getJSONArray("records");
            int harvestedEntities = j*perPage;
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder;
            for(int i=0; i<mediaList.length(); i++){
               try{
                    ImageMetsMods imm = new ImageMetsMods(mediaList.getJSONObject(i));
                    

                    builder = factory.newDocumentBuilder();
                    Document doc = builder.parse(new InputSource(new StringReader(imm.getXML())));

                    RecordType record = new RecordType();
                    MetadataType metadataMets = new MetadataType();
                    metadataMets.setAny(doc.getDocumentElement());                   
                    record.setMetadata(metadataMets);
                    imageMetsModsList.getRecord().add(record);
                    //printXML(doc);

                } catch (ParserConfigurationException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }                
                
            }
            
        }
        return imageMetsModsList;
    }*/

    public HeaderType buildOAIPMH_RecordHeader(String dateStamp, String Identifier){
        HeaderType recordHeader = new HeaderType();
        recordHeader.setDatestamp(dateStamp);
        recordHeader.setIdentifier(Identifier);
        return recordHeader;
    }

    public ResumptionTokenType buildResumptionToken(String resumptionToken, int completeListSize, String from, String until){
        ResumptionTokenType resTokenForResponse = new ResumptionTokenType();
        if(resumptionToken != null && cursorCollector.containsKey(resumptionToken)){
            if(cursorCollector.get(resumptionToken)*100 > completeListSize){
                resTokenForResponse.setValue("");
            }else{
                resTokenForResponse.setValue(resumptionToken);
            } 
            resTokenForResponse.setCompleteListSize(BigInteger.valueOf(completeListSize));
            resTokenForResponse.setCursor(BigInteger.valueOf(cursorCollector.get(resumptionToken)*100));            
            cursorCollector.put(resumptionToken, cursorCollector.get(resumptionToken)+1);
        }else {
            UUID uuid = UUID.randomUUID();
            String resTokenValue = uuid.toString();
            resTokenForResponse.setCompleteListSize(BigInteger.valueOf(completeListSize));
            resTokenForResponse.setCursor(BigInteger.valueOf(100));
            resTokenForResponse.setValue(resTokenValue);
            cursorCollector.put(resTokenValue, 2);
            fromUntilCollector.put(resTokenValue, from + ":" + until);
        }
        return resTokenForResponse;
    }

    public String printXML(Document doc) throws TransformerFactoryConfigurationError, TransformerException{
        Transformer transformer = TransformerFactory.newInstance().newTransformer();
        transformer.setOutputProperty(OutputKeys.INDENT, "yes");
        transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
        // initialize StreamResult with File object to save to file
        StreamResult result = new StreamResult(new StringWriter());
        DOMSource source = new DOMSource(doc);
        transformer.transform(source, result);
        String xmlString = result.getWriter().toString();
        return xmlString;
    }

    public int getTotalHits() {
        return totalHits;
    }

    public void setTotalHits(int totalHits) {
        this.totalHits = totalHits;
    }

    public String getPageNumber() {
        return pageNumber;
    }

    public void setPageNumber(String pageNumber) {
        this.pageNumber = pageNumber;
    }

    public String getPerPage() {
        return perPage;
    }

    public void setPerPage(String perPage) {
        this.perPage = perPage;
    }

    public String getKindID() {
        return kindID;
    }

    public void setKindID(String kindID) {
        this.kindID = kindID;
    }

	public String getFrom() {
		return from;
	}

	public void setFrom(String from) {
		this.from = from;
	}

	public String getUntil() {
		return until;
	}

	public void setUntil(String until) {
		this.until = until;
	}
}
